@@ -14,10 +14,6 @@
  * Any additions, deletions, or changes to the original source files
  * must be clearly indicated in accompanying documentation.
  *
- * If only executable code is distributed, then the accompanying
- * documentation must state that "this software is based in part on the
- * work of the Khronos Group."
- *
  * THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -29,6 +25,7 @@
 
 #include "glvnd_genentry.h"
 #include "utils_misc.h"
+#include "compiler.h"
 
 #include <string.h>
 #include <stdint.h>
@@ -45,7 +42,7 @@
 #define GLX_STUBS_COUNT
 #include "g_glx_dispatch_stub_list.h"
 
-static GLVNDentrypointStub entrypointFunctions[GENERATED_ENTRYPOINT_MAX];
+USED static GLVNDentrypointStub entrypointFunctions[GENERATED_ENTRYPOINT_MAX];
 static char *entrypointNames[GENERATED_ENTRYPOINT_MAX] = {};
 static int entrypointCount = 0;
 
@@ -126,6 +123,10 @@ __asm__(".globl glx_entrypoint_start\n"
         ".balign " U_STRINGIFY(STUB_SIZE) "\n" \
         "glx_entrypoint_start:\n"
 
+#if defined(USE_ARMV7_ASM) && defined(__thumb__)
+        ".arm\n"
+#endif
+
 #define GLX_STUBS_ASM
 #include "g_glx_dispatch_stub_list.h"
 
@@ -133,6 +134,10 @@ __asm__(".globl glx_entrypoint_start\n"
         ".hidden glx_entrypoint_end\n"
         ".balign " U_STRINGIFY(STUB_SIZE) "\n" \
         "glx_entrypoint_end:\n"
+
+#if defined(USE_ARMV7_ASM) && defined(__thumb__)
+        ".thumb\n"
+#endif
 );
 
 static void *DefaultDispatchFunc(void)
