@@ -14,10 +14,6 @@
  * Any additions, deletions, or changes to the original source files
  * must be clearly indicated in accompanying documentation.
  *
- * If only executable code is distributed, then the accompanying
- * documentation must state that "this software is based in part on the
- * work of the Khronos Group."
- *
  * THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -53,6 +49,16 @@
 #define GLX_MINOR_VERSION 4
 #define GLX_VERSION_STRING "1.4"
 
+/*
+ * Older versions of glxproto.h contained a typo where "Attribs" was misspelled.
+ * The typo was fixed in the xorgproto version of glxproto.h, breaking the API.
+ * Work around that here.
+ */
+#if !defined(X_GLXCreateContextAttribsARB) && \
+    defined(X_GLXCreateContextAtrribsARB)
+#define X_GLXCreateContextAttribsARB X_GLXCreateContextAtrribsARB
+#endif
+
 static glvnd_mutex_t clientStringLock = GLVND_MUTEX_INITIALIZER;
 
 /**
@@ -262,6 +268,48 @@ PUBLIC GLXContext glXCreateNewContext(Display *dpy, GLXFBConfig config,
     return context;
 }
 
+static GLXContext glXCreateContextAttribsARB(Display *dpy, GLXFBConfig config,
+        GLXContext share_list, Bool direct, const int *attrib_list)
+{
+    GLXContext context = NULL;
+    __GLXvendorInfo *vendor = NULL;
+
+    if (attrib_list != NULL) {
+        // See if the caller passed in a GLX_SCREEN attribute, and if so, use
+        // that to select a vendor library. This is needed for
+        // GLX_EXT_no_config_context, where we won't have a GLXFBConfig handle.
+        int i;
+        for (i=0; attrib_list[i] != None; i += 2) {
+            if (attrib_list[i] == GLX_SCREEN) {
+                int screen = attrib_list[i + 1];
+                vendor = __glXGetDynDispatch(dpy, screen);
+                if (vendor == NULL) {
+                    __glXSendError(dpy, BadValue, 0,
+                            X_GLXCreateContextAttribsARB, True);
+                    return None;
+                }
+            }
+        }
+    }
+
+    if (vendor == NULL) {
+        // We didn't get a GLX_SCREEN attribute, so look at the config instead.
+        vendor = CommonDispatchFBConfig(dpy, config, X_GLXCreateContextAttribsARB);
+    }
+
+    if (vendor != NULL && vendor->staticDispatch.createContextAttribsARB != NULL) {
+        context = vendor->staticDispatch.createContextAttribsARB(dpy, config, share_list, direct, attrib_list);
+        if (context != NULL) {
+            if (__glXAddVendorContextMapping(dpy, context, vendor) != 0) {
+                vendor->staticDispatch.destroyContext(dpy, context);
+                context = NULL;
+            }
+        }
+    }
+
+    return context;
+}
+
 PUBLIC void glXDestroyContext(Display *dpy, GLXContext context)
 {
     __GLXvendorInfo *vendor;
@@ -1680,6 +1728,7 @@ const __GLXlocalDispatchFunction LOCAL_GLX_DISPATCH_FUNCTIONS[] =
 
         LOCAL_FUNC_TABLE_ENTRY(glXImportContextEXT)
         LOCAL_FUNC_TABLE_ENTRY(glXFreeContextEXT)
+        LOCAL_FUNC_TABLE_ENTRY(glXCreateContextAttribsARB)
 #undef LOCAL_FUNC_TABLE_ENTRY
     { NULL, NULL }
 };
