@@ -14,10 +14,6 @@
  * Any additions, deletions, or changes to the original source files
  * must be clearly indicated in accompanying documentation.
  *
- * If only executable code is distributed, then the accompanying
- * documentation must state that "this software is based in part on the
- * work of the Khronos Group."
- *
  * THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -33,7 +29,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xlibint.h>
 #include <GL/glx.h>
-#include <GL/glxint.h>
 
 #include "GLX_dummy.h"
 #include "glvnd/libglxabi.h"
@@ -49,9 +44,7 @@ static const __GLXapiExports *apiExports = NULL;
  * Dummy context structure.
  */
 typedef struct __GLXcontextRec {
-    GLint beginHit;
-    GLint vertex3fvHit;
-    GLint endHit;
+    GLContextCounts counts;
 } __GLXcontext;
 
 static const int FBCONFIGS_PER_SCREEN = 10;
@@ -67,8 +60,8 @@ static void dummy_glXExampleExtensionFunction(Display *dpy, int screen, int *ret
 static void dispatch_glXExampleExtensionFunction(Display *dpy, int screen, int *retval);
 static void dummy_glXExampleExtensionFunction2(Display *dpy, int screen, int *retval);
 static void dispatch_glXExampleExtensionFunction2(Display *dpy, int screen, int *retval);
-static void dummy_glXMakeCurrentTestResults(GLint req, GLboolean *saw, void **ret);
-static void dispatch_glXMakeCurrentTestResults(GLint req, GLboolean *saw, void **ret);
+static void dummy_glXMakeCurrentTestResults(GLboolean *saw, GLContextCounts *counts);
+static void dispatch_glXMakeCurrentTestResults(GLboolean *saw, GLContextCounts *counts);
 
 enum
 {
@@ -159,10 +152,7 @@ static void          dummy_glXCopyContext           (Display *dpy,
 static GLXContext CommonCreateContext(Display *dpy, int screen)
 {
     if (screen >= 0) {
-        __GLXcontext *context = malloc(sizeof(*context));
-        context->beginHit = 0;
-        context->vertex3fvHit = 0;
-        context->endHit = 0;
+        __GLXcontext *context = calloc(1, sizeof(*context));
         return context;
     } else {
         return NULL;
@@ -507,7 +497,7 @@ static void dummy_glBegin (void)
     GLXContext ctx = apiExports->getCurrentContext();
     assert(ctx);
 
-    ctx->beginHit++;
+    ctx->counts.beginCount++;
 }
 
 static void dummy_glVertex3fv(GLfloat *v)
@@ -515,7 +505,7 @@ static void dummy_glVertex3fv(GLfloat *v)
     GLXContext ctx = apiExports->getCurrentContext();
     assert(ctx);
 
-    ctx->vertex3fvHit++;
+    ctx->counts.vertex3fvCount++;
 }
 
 static void dummy_glEnd (void)
@@ -523,33 +513,19 @@ static void dummy_glEnd (void)
     GLXContext ctx = apiExports->getCurrentContext();
     assert(ctx);
 
-    ctx->endHit++;
+    ctx->counts.endCount++;
 }
 
-static void dummy_glXMakeCurrentTestResults(GLint req, GLboolean *saw, void **ret)
+static void dummy_glXMakeCurrentTestResults(GLboolean *saw, GLContextCounts *counts)
 {
     GLXContext ctx = apiExports->getCurrentContext();
     assert(ctx);
 
     *saw = GL_TRUE;
-    switch (req) {
-    case GL_MC_FUNCTION_COUNTS:
-        {
-            GLint *data = (GLint *)malloc(3 * sizeof(GLint));
-            data[0] = ctx->beginHit;
-            data[1] = ctx->vertex3fvHit;
-            data[2] = ctx->endHit;
-            *ret = (void *)data;
-        }
-        break;
-    case GL_MC_LAST_REQ:
-    default:
-        *ret = NULL;
-        break;
-    }
+    *counts = ctx->counts;
 }
 
-static void dispatch_glXMakeCurrentTestResults(GLint req, GLboolean *saw, void **ret)
+static void dispatch_glXMakeCurrentTestResults(GLboolean *saw, GLContextCounts *counts)
 {
     __GLXvendorInfo *dynDispatch;
     PFNGLXMAKECURRENTTESTRESULTSPROC func;
@@ -563,7 +539,7 @@ static void dispatch_glXMakeCurrentTestResults(GLint req, GLboolean *saw, void *
     func = (PFNGLXMAKECURRENTTESTRESULTSPROC)
         apiExports->fetchDispatchEntry(dynDispatch, index);
     if (func) {
-        func(req, saw, ret);
+        func(saw, counts);
     }
 }
 
